//
//  AppManager.swift
//  SampleAppLiveVideo
//
//  Created by Nutchaphon Rewik on 21/10/2563 BE.
//  Copyright © 2563 Amity. All rights reserved.
//

import Foundation

import AmitySDK
import AmityVideoPlayerKit

/// Manage application context and services.
class AppManager {
    
    /// Access the singleton object via this variable.
    static let shared = AppManager()
    
    /// Repository of stream to be used all over the app.
    private(set) var streamRepository: AmityStreamRepository?
    private(set) var fileRepository: AmityFileRepository?
    
    private(set) var client: AmityClient?
    
    private var getStreamToken: AmityNotificationToken?
    
    ///
    private init() {}
    
    deinit {
        getStreamToken = nil
    }
    
    func setup(completion: AmityRequestCompletion?) {
        // 0. For testing
        // 1. Setup client
        if let apiKey = getAPIKey() {
            let urlInfo = getUrlInfo()
            
            if let httpUrl = urlInfo.0, let socketUrl = urlInfo.1, let mqttUrl = urlInfo.2, !httpUrl.isEmpty, !socketUrl.isEmpty, !mqttUrl.isEmpty {
                client = try? AmityClient(apiKey: apiKey, endpoint: AmityEndpoint(httpUrl: httpUrl, rpcUrl: socketUrl, mqttHost: mqttUrl))
            } else {
                client = try? AmityClient(apiKey: apiKey)
            }
        } else {
            print("The app is initialized without API Key.")
            client = try? AmityClient(apiKey: "NO_API_KEY")
        }
        // 2. Setup client for the player, to make it works.
        AmityVideoPlayer.setup(client: client!)
        // 3. Register device with a user.
        client?.login(userId: "test_user", displayName: "Tester SDK", authToken: nil, completion: completion)
        // 4. Set up stream repository
        streamRepository = AmityStreamRepository(client: client!)
        fileRepository = AmityFileRepository(client: client!)
        //
        requestNotificationPermission()
    }
    
    func sendDeviceTokenToServer(token: Data) {
        
        // Transform deviceToken into a raw string, before sending to AmityChatSDK server.
        let tokenParts: [String] = token.map { data in String(format: "%02.2hhx", data) }
        let tokenString: String = tokenParts.joined()
        
        client?.registerDeviceForPushNotification(withDeviceToken: tokenString, completion: { success, error in
            if let error = error {
                print("Unable to register device for push notification: \(error.localizedDescription)")
                return
            }
            print("Successfully register device for push notification")
        })
        
    }
    
    func getAPIKey() -> String? {
        return UserDefaults.standard.string(forKey: "api_key")
    }
    
    func saveAPIKey(_ apiKey: String?) {
        if let apiKey = apiKey, !apiKey.isEmpty {
            UserDefaults.standard.setValue(apiKey, forKey: "api_key")
        } else {
            UserDefaults.standard.removeObject(forKey: "api_key")
        }
    }
    
    func saveUrlInfo(httpURL: String, rpcURL: String, mqttURL: String) {
        UserDefaults.standard.setValue(httpURL, forKey: "live_video_http_url")
        UserDefaults.standard.setValue(rpcURL, forKey: "live_video_rpc_url")
        UserDefaults.standard.setValue(mqttURL, forKey: "live_video_mqtt_url")
    }
    
    func getUrlInfo() -> (String?,String?,String?) {
        let httpUrl = UserDefaults.standard.string(forKey: "live_video_http_url")
        let rpcUrl = UserDefaults.standard.string(forKey: "live_video_rpc_url")
        let mqttUrl = UserDefaults.standard.string(forKey: "live_video_mqtt_url")
        
        return (httpUrl, rpcUrl, mqttUrl)
    }
    
    private func requestNotificationPermission() {
        let center = UNUserNotificationCenter.current()
        center.requestAuthorization(options: [.alert, .sound, .badge]) { granted, error in
            guard granted else {
                return
            }
        }
    }
    
    private var rootViewController: RootViewController {
        let sceneDelegate = UIApplication.shared.connectedScenes
            .first!.delegate as! SceneDelegate
        return sceneDelegate.window!.rootViewController as! RootViewController
    }
    
    func presentLiveVideo(forStreamId streamId: String) {
        
        guard let streamProxy = AppManager.shared.streamRepository?.getStreamById(streamId) else {
            return
        }
        
        // Ensure the stream object is fetched.
        getStreamToken = streamProxy.observe { [weak self] (proxy, error) in
            if let error = error {
                print("streamProxy.observe error: \(error.localizedDescription)")
                return
            }
            guard
                let strongSelf = self,
                let stream = proxy.object else {
                    return
            }
            // `observe` will be called multiple times.
            // In this case, we want to make sure, we get the last fetched object from server.
            // So we check `dataStatus == .fresh`, and execute the logic.
            guard proxy.dataStatus == .fresh else {
                return
            }
            
            // After we get the fresh object, we are no longer interest in any future updates, so we delete the token.
            strongSelf.getStreamToken = nil
            
            // Present this live stream on the new page from root view controller.
            let playController = PlayerViewController.make()
            playController.streamToWatch = stream
            strongSelf.rootViewController.popViewController(animated: true)
            strongSelf.rootViewController.pushViewController(playController, animated: true)
            
        }
        
    }
    
}

extension NSNotification.Name {
    static let presentLiveVideo = NSNotification.Name("presentLiveVideo")
    static let updateUser = NSNotification.Name("updateUser")
}
